==> matrices-0.1.0.tgz: package/lib/matrices/matrix.js <==
var Matrix;
var __slice = Array.prototype.slice;
module.exports = Matrix = (function() {
  function Matrix(rows, cols, items) {
    if (items == null) {
      items = [];
    }
    this.rows = rows;
    this.cols = cols;
    this.items = items;
  }
  Matrix.identity = function(n) {
    var i, items, j;
    items = [];
    for (i = 0; 0 <= n ? i < n : i > n; 0 <= n ? i++ : i--) {
      for (j = 0; 0 <= n ? j < n : j > n; 0 <= n ? j++ : j--) {
        items.push(i === j ? 1 : 0);
      }
    }
    return new Matrix(n, n, items);
  };
  Matrix.prototype.index = function(i, j) {
    return i * this.cols + j;
  };
  Matrix.prototype.get = function(i, j) {
    return this.items[this.index(i, j)];
  };
  Matrix.prototype.project = function() {
    var fn, fns, ind, items, row, _i, _len, _ref;
    fns = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    items = [];
    for (row = 0, _ref = this.rows; 0 <= _ref ? row < _ref : row > _ref; 0 <= _ref ? row++ : row--) {
      for (_i = 0, _len = fns.length; _i < _len; _i++) {
        fn = fns[_i];
        ind = row * this.cols;
        items.push(fn(this.items.slice(ind, ind + this.cols)));
      }
    }
    return new Matrix(this.rows, fns.length, items);
  };
  Matrix.column = function(n) {
    return function(arr) {
      return arr[n];
    };
  };
  Matrix.prototype.proj = Matrix.prototype.project;
  Matrix.prototype.transpose = function() {
    var i, items, j, _ref, _ref2;
    items = [];
    for (i = 0, _ref = this.rows; 0 <= _ref ? i < _ref : i > _ref; 0 <= _ref ? i++ : i--) {
      for (j = 0, _ref2 = this.cols; 0 <= _ref2 ? j < _ref2 : j > _ref2; 0 <= _ref2 ? j++ : j--) {
        items.push(this.items[j * this.rows + i]);
      }
    }
    return new Matrix(this.cols, this.rows, items);
  };
  Matrix.prototype.transp = Matrix.prototype.transpose;
  Matrix.prototype.t = Matrix.prototype.transpose;
  Matrix.prototype.clone = function() {
    return new Matrix(this.rows, this.cols, this.items.slice(0));
  };
  Matrix.prototype.inverse = function() {
    var i, ii, ind, inv, j, k, lead, leadVal, m, res, row, size, temp, test, testVal, _ref, _ref2, _ref3, _ref4;
    if (this.rows !== this.cols) {
      throw new Error('Dimensionality mismatch');
    }
    m = this.clone();
    size = this.rows;
    row = (function() {
      var _results;
      _results = [];
      for (i = 0; i < size; i += 1) {
        _results.push(i);
      }
      return _results;
    })();
    inv = Matrix.identity(size);
    for (i = 0; i < size; i += 1) {
      lead = i;
      leadVal = m.get(row[lead], i);
      for (j = _ref = i + 1; j < size; j += 1) {
        test = j;
        testVal = m.get(row[test], i);
        if (Math.abs(testVal) > Math.abs(leadVal)) {
          leadVal = testVal;
          lead = j;
        }
      }
      temp = row[i];
      row[i] = row[lead];
      row[lead] = temp;
      for (j = 0; j < size; j += 1) {
        ind = m.index(row[i], j);
        m.items[ind] /= leadVal;
        inv.items[ind] /= leadVal;
      }
      for (ii = _ref2 = i + 1; ii < size; ii += 1) {
        k = m.get(row[ii], i);
        for (j = 0; j < size; j += 1) {
          ind = m.index(row[ii], j);
          m.items[ind] -= k * m.get(row[i], j);
          inv.items[ind] -= k * inv.get(row[i], j);
        }
      }
    }
    for (i = _ref3 = size - 1; i >= 0; i += -1) {
      for (ii = _ref4 = i - 1; ii >= 0; ii += -1) {
        k = m.get(row[ii], i);
        for (j = 0; j < size; j += 1) {
          ind = m.index(row[ii], j);
          m.items[ind] -= k * m.get(row[i], j);
          inv.items[ind] -= k * inv.get(row[i], j);
        }
      }
    }
    res = new Matrix(size, size);
    for (i = 0; i < size; i += 1) {
      for (j = 0; j < size; j += 1) {
        res.items[res.index(i, j)] = inv.items[inv.index(row[i], j)];
      }
    }
    return res;
  };
  Matrix.prototype.inv = Matrix.prototype.inverse;
  Matrix.prototype.negate = function() {
    var item;
    return new Matrix(this.rows, this.cols, (function() {
      var _i, _len, _ref, _results;
      _ref = this.items;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        _results.push(-item);
      }
      return _results;
    }).call(this));
  };
  Matrix.prototype.neg = Matrix.prototype.negate;
  Matrix.prototype.add = function(other) {
    var i, items, _ref;
    if (this.rows !== other.rows || this.cols !== other.cols) {
      throw new Error('Dimensionality mismatch');
    }
    items = [];
    for (i = 0, _ref = this.items.length; 0 <= _ref ? i < _ref : i > _ref; 0 <= _ref ? i++ : i--) {
      items.push(this.items[i] + other.items[i]);
    }
    return new Matrix(this.rows, this.cols, items);
  };
  Matrix.prototype.subtract = function(other) {
    return this.add(other.neg());
  };
  Matrix.prototype.sub = Matrix.prototype.subtract;
  Matrix.prototype.multiply = function(other) {
    var i, items, j, k, sum, _ref, _ref2, _ref3;
    if (this.cols !== other.rows) {
      throw new Error('Dimensionality mismatch');
    }
    items = [];
    for (i = 0, _ref = this.rows; 0 <= _ref ? i < _ref : i > _ref; 0 <= _ref ? i++ : i--) {
      for (j = 0, _ref2 = other.cols; 0 <= _ref2 ? j < _ref2 : j > _ref2; 0 <= _ref2 ? j++ : j--) {
        sum = 0;
        for (k = 0, _ref3 = this.cols; 0 <= _ref3 ? k < _ref3 : k > _ref3; 0 <= _ref3 ? k++ : k--) {
          sum += this.get(i, k) * other.get(k, j);
        }
        items[i * other.cols + j] = sum;
      }
    }
    return new Matrix(this.rows, other.cols, items);
  };
  Matrix.prototype.mul = Matrix.prototype.multiply;
  Matrix.prototype.divide = function(other) {
    return this.mul(other.inverse());
  };
  Matrix.prototype.div = Matrix.prototype.divide;
  Matrix.prototype.norm = function() {
    var item, norm, _i, _len, _ref;
    norm = 0;
    _ref = this.items;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      item = _ref[_i];
      norm += item * item;
    }
    return norm;
  };
  Matrix.prototype.toString = function() {
    var ind, item, items, row, s, _ref;
    s = '';
    for (row = 0, _ref = this.rows; row < _ref; row += 1) {
      ind = row * this.cols;
      items = (function() {
        var _i, _len, _ref2, _results;
        _ref2 = this.items.slice(ind, ind + this.cols);
        _results = [];
        for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
          item = _ref2[_i];
          _results.push(item.toFixed(2));
        }
        return _results;
      }).call(this);
      s += items.join(', ') + "\n";
    }
    return s;
  };
  return Matrix;
})();

==> matrices-0.2.0.tgz: package/lib/matrices/matrix.js <==
var Matrix;
var __slice = Array.prototype.slice;
module.exports = Matrix = (function() {
  var decompose;
  function Matrix(rows, cols, items) {
    if (items == null) {
      items = [];
    }
    this.rows = rows;
    this.cols = cols;
    this.items = items;
  }
  Matrix.identity = function(n) {
    var i, items, j;
    items = [];
    for (i = 0; 0 <= n ? i < n : i > n; 0 <= n ? i++ : i--) {
      for (j = 0; 0 <= n ? j < n : j > n; 0 <= n ? j++ : j--) {
        items.push(i === j ? 1 : 0);
      }
    }
    return new Matrix(n, n, items);
  };
  Matrix.prototype.index = function(i, j) {
    return i * this.cols + j;
  };
  Matrix.prototype.get = function(i, j) {
    return this.items[this.index(i, j)];
  };
  Matrix.prototype.project = function() {
    var fn, fns, ind, items, row, _i, _len, _ref;
    fns = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    items = [];
    for (row = 0, _ref = this.rows; 0 <= _ref ? row < _ref : row > _ref; 0 <= _ref ? row++ : row--) {
      for (_i = 0, _len = fns.length; _i < _len; _i++) {
        fn = fns[_i];
        ind = row * this.cols;
        items.push(fn(this.items.slice(ind, ind + this.cols)));
      }
    }
    return new Matrix(this.rows, fns.length, items);
  };
  Matrix.prototype.proj = Matrix.prototype.project;
  Matrix.prototype.select = Matrix.prototype.project;
  Matrix.prototype.transpose = function() {
    var i, items, j, _ref, _ref2;
    items = [];
    for (i = 0, _ref = this.rows; 0 <= _ref ? i < _ref : i > _ref; 0 <= _ref ? i++ : i--) {
      for (j = 0, _ref2 = this.cols; 0 <= _ref2 ? j < _ref2 : j > _ref2; 0 <= _ref2 ? j++ : j--) {
        items[j * this.rows + i] = this.get(i, j);
      }
    }
    return new Matrix(this.cols, this.rows, items);
  };
  Matrix.prototype.transp = Matrix.prototype.transpose;
  Matrix.prototype.t = Matrix.prototype.transpose;
  Matrix.prototype.clone = function() {
    return new Matrix(this.rows, this.cols, this.items.slice(0));
  };
  Matrix.prototype.augment = function(other) {
    var otherT, selfT, _ref;
    if (this.rows !== other.rows) {
      throw new Error('Dimensionality mismatch');
    }
    selfT = this.transpose();
    otherT = other.transpose();
    selfT.rows += otherT.rows;
    (_ref = selfT.items).push.apply(_ref, otherT.items);
    return selfT.transpose();
  };
  Matrix.prototype.sliceRows = function(begin, end) {
    if (begin < 0) {
      begin += this.rows;
    }
    if (begin < 0) {
      begin = 0;
    }
    if (end == null) {
      end = this.rows;
    }
    if (end > this.rows) {
      end = this.rows;
    } else if (end < 0) {
      end += this.rows;
    }
    if (end < begin) {
      end = begin;
    }
    return new Matrix(end - begin, this.cols, this.items.slice(begin * this.cols, end * this.cols));
  };
  Matrix.prototype.sliceCols = function(begin, end) {
    if (end == null) {
      end = this.cols;
    }
    return this.transpose().sliceRows(begin, end).transpose();
  };
  Matrix.prototype.slice = function(beginCols, beginRows, endCols, endRows) {
    if (beginRows == null) {
      beginRows = 0;
    }
    if (endCols == null) {
      endCols = this.cols;
    }
    if (endRows == null) {
      endRows = this.rows;
    }
    return this.sliceRows(beginRows, endRows).sliceCols(beginCols, endCols);
  };
  decompose = function() {
    var det, i, ii, ind, j, k, lower, pivot, pivotVal, row, size, temp, test, testVal, upper, _ref, _ref2;
    if (this.rows !== this.cols) {
      throw new Error('Dimensionality mismatch');
    }
    size = this.rows;
    upper = this.clone();
    lower = Matrix.identity(size);
    row = (function() {
      var _results;
      _results = [];
      for (i = 0; i < size; i += 1) {
        _results.push(i);
      }
      return _results;
    })();
    det = 1;
    for (i = 0; i < size; i += 1) {
      pivot = i;
      pivotVal = upper.get(row[pivot], i);
      for (j = _ref = i + 1; j < size; j += 1) {
        test = j;
        testVal = upper.get(row[test], i);
        if (Math.abs(testVal) > Math.abs(pivotVal)) {
          pivotVal = testVal;
          pivot = test;
        }
      }
      det *= pivotVal;
      if (pivotVal === 0) {
        return {
          det: 0
        };
      }
      if (i !== pivot) {
        temp = row[i];
        row[i] = row[pivot];
        row[pivot] = temp;
        det = -det;
      }
      for (j = 0; j < size; j += 1) {
        ind = upper.index(row[i], j);
        upper.items[ind] /= pivotVal;
        lower.items[ind] /= pivotVal;
      }
      for (ii = _ref2 = i + 1; ii < size; ii += 1) {
        k = upper.get(row[ii], i);
        for (j = 0; j < size; j += 1) {
          ind = upper.index(row[ii], j);
          upper.items[ind] -= k * upper.get(row[i], j);
          lower.items[ind] -= k * lower.get(row[i], j);
        }
      }
    }
    return {
      upper: upper,
      lower: lower,
      perm: row,
      det: det
    };
  };
  Matrix.prototype.determinant = function() {
    return decompose.call(this).det;
  };
  Matrix.prototype.det = Matrix.prototype.determinant;
  Matrix.prototype.inverse = function() {
    var det, i, ii, ind, j, k, lower, perm, res, size, upper, _ref, _ref2, _ref3;
    _ref = decompose.call(this), upper = _ref.upper, lower = _ref.lower, perm = _ref.perm, det = _ref.det;
    if (det < 1e-8) {
      throw new Error('Matrix is not invertible or is ill-conditioned');
    }
    size = this.rows;
    for (i = _ref2 = size - 1; i >= 0; i += -1) {
      for (ii = _ref3 = i - 1; ii >= 0; ii += -1) {
        k = upper.get(perm[ii], i);
        for (j = 0; j < size; j += 1) {
          ind = upper.index(perm[ii], j);
          upper.items[ind] -= k * upper.get(perm[i], j);
          lower.items[ind] -= k * lower.get(perm[i], j);
        }
      }
    }
    res = new Matrix(size, size);
    for (i = 0; i < size; i += 1) {
      for (j = 0; j < size; j += 1) {
        res.items[res.index(i, j)] = lower.items[lower.index(perm[i], j)];
      }
    }
    return res;
  };
  Matrix.prototype.inv = Matrix.prototype.inverse;
  Matrix.prototype.negate = function() {
    var item;
    return new Matrix(this.rows, this.cols, (function() {
      var _i, _len, _ref, _results;
      _ref = this.items;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        _results.push(-item);
      }
      return _results;
    }).call(this));
  };
  Matrix.prototype.neg = Matrix.prototype.negate;
  Matrix.prototype.add = function(other) {
    var i, items, _ref;
    if (this.rows !== other.rows || this.cols !== other.cols) {
      throw new Error('Dimensionality mismatch');
    }
    items = [];
    for (i = 0, _ref = this.items.length; 0 <= _ref ? i < _ref : i > _ref; 0 <= _ref ? i++ : i--) {
      items.push(this.items[i] + other.items[i]);
    }
    return new Matrix(this.rows, this.cols, items);
  };
  Matrix.prototype.subtract = function(other) {
    return this.add(other.negate());
  };
  Matrix.prototype.sub = Matrix.prototype.subtract;
  Matrix.prototype.multiply = function(other) {
    var i, items, j, k, sum, _ref, _ref2, _ref3;
    if (this.cols !== other.rows) {
      throw new Error('Dimensionality mismatch');
    }
    items = [];
    for (i = 0, _ref = this.rows; 0 <= _ref ? i < _ref : i > _ref; 0 <= _ref ? i++ : i--) {
      for (j = 0, _ref2 = other.cols; 0 <= _ref2 ? j < _ref2 : j > _ref2; 0 <= _ref2 ? j++ : j--) {
        sum = 0;
        for (k = 0, _ref3 = this.cols; 0 <= _ref3 ? k < _ref3 : k > _ref3; 0 <= _ref3 ? k++ : k--) {
          sum += this.get(i, k) * other.get(k, j);
        }
        items[i * other.cols + j] = sum;
      }
    }
    return new Matrix(this.rows, other.cols, items);
  };
  Matrix.prototype.mul = Matrix.prototype.multiply;
  Matrix.prototype.divide = function(other) {
    return this.mul(other.inverse());
  };
  Matrix.prototype.div = Matrix.prototype.divide;
  Matrix.prototype.norm = function() {
    var item, norm, _i, _len, _ref;
    norm = 0;
    _ref = this.items;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      item = _ref[_i];
      norm += item * item;
    }
    return norm;
  };
  Matrix.prototype.toString = function() {
    var ind, item, items, row, s, _ref;
    s = '';
    for (row = 0, _ref = this.rows; row < _ref; row += 1) {
      ind = row * this.cols;
      items = (function() {
        var _i, _len, _ref2, _results;
        _ref2 = this.items.slice(ind, ind + this.cols);
        _results = [];
        for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
          item = _ref2[_i];
          _results.push(item.toFixed(2));
        }
        return _results;
      }).call(this);
      s += items.join(', ') + "\n";
    }
    return s;
  };
  return Matrix;
})();
